# (c) Copyright 2020 Trent Hauck
# All Rights Reserved
"""Main entrypoint for opttt."""

import argparse
import sys

import pydantic


def _add_fields_to_parser(obj, parser):
    for field_name, field in obj.__fields__.items():

        extra = field.field_info.extra

        if field.required and "env" not in extra:
            parser.add_argument(field.alias, type=field.type_, help=field.field_info.description)
        else:
            parser.add_argument(
                f"--{field.alias}",
                type=field.type_,
                help=field.field_info.description,
                default=field.default,
            )


class Opttt(pydantic.BaseSettings):
    @classmethod
    def from_args(cls, arguments=sys.argv[1:]):

        if hasattr(cls, "Config"):
            description = cls.Config.description
        else:
            description = None

        parser = argparse.ArgumentParser(description=description)

        _add_fields_to_parser(cls, parser)

        arguments = parser.parse_args(arguments)
        return cls.parse_obj(vars(arguments))



"""Tests for `opttt` package."""

import pathlib
import uuid

from pydantic import Field

from opttt import Opttt


def test_print():
    """Test that the print command is properly parsed."""

    class Options(Opttt):

        a: int
        b: int

        class Config:
            name: str = "The application."
            description: str = "The description."

    options = Options.from_args(["1", "2"])

    assert options.a == 1
    assert options.b == 2


def test_optional():
    """Test optional arguments work."""

    class DefaultOptions(Opttt):

        b: int
        a: int = Field(1, description="Help for A.")

        class Config:
            name: str = "The application."
            description: str = "The description."

    options = DefaultOptions.from_args(["2"])
    assert options.a == 1

    options = DefaultOptions.from_args(["2", "--a", "2"])
    assert options.a == 2


def test_complex_char():
    """Test a complex type."""

    class DefaultOptions(Opttt):

        b: pathlib.Path
        a: uuid.UUID

        class Config:
            name: str = "The application."
            description: str = "The description."

    options = DefaultOptions.from_args(["./", "0321fd6a-7374-4536-a5e3-df3673d3427b"])
    assert options.a == uuid.UUID("0321fd6a-7374-4536-a5e3-df3673d3427b")
    assert options.b == pathlib.Path("./")
